#include <dos.h>

REGPACK regs;

struct MouseState
{
  int x, y, buttons;
};

struct CursorShape
{
  unsigned AndMask[16];
  unsigned XorMask[16];
  int hot_x, hot_y;
};

int MouseReset(int &buttons)
{
  regs.r_ax = 0;
  intr(0x33, &regs);
  buttons = regs.r_bx;
  return (regs.r_ax != 0xFFFF);
}

void MouseShow()
{
  regs.r_ax = 1;
  intr(0x33, &regs);
}

void MouseHide()
{
  regs.r_ax = 2;
  intr(0x33, &regs);
}

void MouseRead(MouseState &state)
{
  regs.r_ax = 3;
  intr(0x33, &regs);
  state.x = regs.r_cx;
  state.y = regs.r_dx;
  state.buttons = regs.r_bx;
}

void MouseSetPos(int x, int y)
{
  regs.r_cx = x;
  regs.r_dx = y;
  regs.r_ax = 4;
  intr(0x33, &regs);
}

void MouseGetPressedButtons(int button, int &buttons, int &count, int &x, int &y)
{
  regs.r_ax = 5;
  regs.r_bx = button;
  intr(0x33, &regs);
  buttons = regs.r_ax;
  count = regs.r_bx;
  x = regs.r_cx;
  y = regs.r_dx;
}

void MouseGetReleasedButtons(int button, int &buttons, int &count, int &x, int &y)
{
  regs.r_ax = 6;
  regs.r_bx = button;
  intr(0x33, &regs);
  buttons = regs.r_ax;
  count = regs.r_bx;
  x = regs.r_cx;
  y = regs.r_dx;
}

void MouseSetXRange(int x_min, int x_max)
{
  regs.r_ax = 7;
  regs.r_cx = x_min;
  regs.r_dx = x_max;
  intr(0x33, &regs);
}

void MouseSetYRange(int y_min, int y_max)
{
  regs.r_ax = 8;
  regs.r_cx = y_min;
  regs.r_dx = y_max;
  intr(0x33, &regs);
}

void SetMouseShape(CursorShape shape)
{
  regs.r_bx = shape.hot_x;
  regs.r_cx = shape.hot_y;
  regs.r_dx = FP_OFF(&shape);
  regs.r_ax = 9;
  intr(0x33, &regs);
}

void SetMouseCursor(int and_mask, int or_mask)
{
  regs.r_ax = 0xA;
  regs.r_bx = 0;
  regs.r_cx = and_mask;
  regs.r_dx = or_mask;
  intr(0x33, &regs);
}

void ReadMouseCount(int &dx, int &dy)
{
  regs.r_ax = 0xB;
  intr(0x33, &regs);
  dx = regs.r_cx;
  dy = regs.r_dx;
}

void SetHideRegion(int x1, int y1, int x2, int y2)
{
  regs.r_ax = 0x10;
  regs.r_cx = x1;
  regs.r_dx = y1;
  regs.r_si = x2;
  regs.r_di = y2;
  intr(0x33, &regs);
}

void SetDoubleSpeed(int speed)
{
  regs.r_ax = 0x13;
  regs.r_dx = speed;
  intr(0x33, &regs);
}

void SetMouseSensitive(int mikki_x, int mikki_y, int speed)
{
  regs.r_ax = 0x1A;
  regs.r_bx = mikki_x;
  regs.r_bx = mikki_y;
  regs.r_dx = speed;
  intr(0x33, &regs);
}

void GetMouseSensitive(int &mikki_x, int &mikki_y, int &speed)
{
  regs.r_ax = 0x1B;
  intr(0x33, &regs);
  mikki_x = regs.r_bx;
  mikki_y = regs.r_bx;
  speed = regs.r_dx;
}

void SetInterruptFreq(int flags)
{
  regs.r_ax = 0x1C;
  regs.r_bx = flags;
  intr(0x33, &regs);
}

void GetMouseInfo(int &version, unsigned char &irq_number)
{
  regs.r_ax = 0x24;
  intr(0x33, &regs);
  version = regs.r_bx;
  irq_number = regs.r_cx & 0xFF;
}
